;----------------------------------------------------------------------------------------------------
; Procedure for copying a string to another for max lenght specified
;----------------------------------------------------------------------------------------------------

AssignStr  PROC PtrDest:DWORD, PtrSource:DWORD, MaxLen:DWORD

  mov   edi, PtrDest
  mov   esi, PtrSource
  mov   ecx, MaxLen
  xor   ebx, ebx 

  .WHILE (ebx != ecx)

       mov    al, BYTE PTR [esi]
       mov [edi], al
       
       .BREAK  .IF ( al == 0)

       inc    esi
       inc    edi

       inc    ebx

  .ENDW 

  xor  edx, edx
    
  mov  [edi], dl  

  ret 


AssignStr  ENDP

;----------------------------------------------------------------------------------------------------
; Procedure for formatting a string with thousand comma/point separator
;----------------------------------------------------------------------------------------------------


FormatNum PROC PtrNumber:DWORD

  invoke GetNumberFormat, NULL, 0, PtrNumber, offset NumFmt, offset BufNum, 255

  ret  

FormatNum ENDP

;----------------------------------------------------------------------------------------------------

 
; ---------------------------------------------------------------------------
; The size, title and buffer of the Console are set.
; Max Console Size is set to 34 Rows and 90 Cols.
; ---------------------------------------------------------------------------

ConsoleSize PROC

    INVOKE AllocConsole

    INVOKE GetStdHandle, STD_INPUT_HANDLE
    mov rHnd,eax

    INVOKE GetStdHandle, STD_OUTPUT_HANDLE
    mov wHnd,eax 

    INVOKE SetConsoleTitle, ADDR ConTitle
  
    CALL   HideTheCursor

    mov windowSize.Left, Zero
    mov windowSize.Right, (MaxCols -1)
    mov windowSize.Top, Zero
    mov windowSize.Bottom, (MaxRows -1)

    
    mov bufferSize.x, MaxCols
    mov bufferSize.y, MaxRows

    INVOKE SetConsoleWindowInfo, wHnd, TRUE, ADDR windowSize

    INVOKE SetConsoleScreenBufferSize, wHnd, DWORD PTR bufferSize

    ret

ConsoleSize ENDP

; ---------------------------------------------------------------------------
; Reads the Screen File Corner Positions.
; ---------------------------------------------------------------------------
;    The initial 4 bytes contain:
;
;    RowInitial, RowFinal, ColInitial, ColFinal of the Screen File to display
; ---------------------------------------------------------------------------
 

GetConsole PROC 


    movzx eax, RowFinal
    mov   R2,  ax
    inc   eax
    sub   al,  RowInitial

    mov   BufRows, ax

    movzx ebx, ColFinal
    mov   C2,  bx    
    inc   ebx
    sub   bl,  ColInitial

    mov   BufCols, bx

    movzx ax, RowInitial  
    mov   R1, ax

    dec   R1
    dec   R2
    movzx ax, ColInitial
    mov   C1, ax 
    dec   C1   
    dec   C2

    mov   ebx, BytesToRead

    mov   BufChars, ebx

    ret 
    
GetConsole ENDP

;----------------------------------------------------------------------------
; The Cursor is made visible
;----------------------------------------------------------------------------

ShowTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,TRUE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
ShowTheCursor ENDP

;----------------------------------------------------------------------------
; The Cursor is made invisible
;----------------------------------------------------------------------------
    


HideTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,FALSE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
HideTheCursor ENDP

;----------------------------------------------------------------------------
; After the Screen File has been read, the parameters to display it are set
;----------------------------------------------------------------------------

InitProc PROC

 
    mov ax, C1
    mov windowSize.Left, ax
    mov ax, C2
    mov windowSize.Right, ax
    mov ax, R1
    mov windowSize.Top, ax
    mov ax, R2
    mov windowSize.Bottom, ax
   
    mov ax, BufRows
    mov bx, BufCols
    
    mov bufferSize.x, bx
    mov bufferSize.y, ax

    mov startPoint.x, Zero
    mov startPoint.y, Zero


    ret

InitProc ENDP

;----------------------------------------------------------------------------
; The Screen File is displayed on the Console 
;----------------------------------------------------------------------------

DisplayFmt PROC szFmtName:DWORD 

    INVOKE WriteConsoleOutput, wHnd, szFmtName, DWORD PTR bufferSize, 
           DWORD PTR startPoint, offset windowSize 

    ret

DisplayFmt ENDP

;----------------------------------------------------------------------------
; The Screen File is saved for future use 
;----------------------------------------------------------------------------

SaveScreen PROC szFmtName:DWORD 

    INVOKE ReadConsoleOutput, wHnd, szFmtName, DWORD PTR bufferSize, 
           DWORD PTR startPoint, offset windowSize 

    ret

SaveScreen ENDP


;----------------------------------------------------------------------------
; Area of the Screen is saved for future use 
;----------------------------------------------------------------------------

SaveArea PROC szFmtName:DWORD, RowStart:WORD, ColStart:WORD,
                               RowEnd:WORD,   ColEnd:WORD 


; ---------------------------------------------------------------------------

    mov ax, ColStart
    mov TestAreaWindowSize.Left, ax

    mov ax, RowStart
    mov TestAreaWindowSize.Top, ax

    mov ax, ColEnd
    mov TestAreaWindowSize.Right, ax

    mov ax, RowEnd
    mov TestAreaWindowSize.Bottom, ax

    
    mov TestAreaBufferSize.x, TestAreaMaxCols
    mov TestAreaBufferSize.y, TestAreaMaxRows

    INVOKE ReadConsoleOutput, wHnd, szFmtName, DWORD PTR TestAreaBufferSize, 
           DWORD PTR startPoint, offset TestAreaWindowSize 

    ret

SaveArea ENDP

;----------------------------------------------------------------------------
; Area of the Screen is displayed  
;----------------------------------------------------------------------------

DisplayArea PROC szFmtName:DWORD, RowStart:WORD, ColStart:WORD,
                                  RowEnd:WORD,   ColEnd:WORD 


; ---------------------------------------------------------------------------

    mov ax, ColStart
    mov TestAreaWindowSize.Left, ax

    mov ax, RowStart
    mov TestAreaWindowSize.Top, ax

    mov ax, ColEnd
    mov TestAreaWindowSize.Right, ax

    mov ax, RowEnd
    mov TestAreaWindowSize.Bottom, ax

    
    mov TestAreaBufferSize.x, TestAreaMaxCols
    mov TestAreaBufferSize.y, TestAreaMaxRows

    INVOKE WriteConsoleOutput, wHnd, szFmtName, DWORD PTR TestAreaBufferSize, 
           DWORD PTR startPoint, offset TestAreaWindowSize 

    ret

DisplayArea ENDP


; -----------------------------------------------------------
; This procedure reads the column and row count, multiplies
; them together to get the number of characters that will fit
; onto the screen, writes that number of blank spaces to the
; screen, set the screen buffer attributes and reposition
; the prompt at position 0,0.
; -----------------------------------------------------------

ClearScreen proc



    LOCAL hOutPut:DWORD
    LOCAL noc    :DWORD
    LOCAL cnt    :DWORD
    LOCAL sbi    :CONSOLE_SCREEN_BUFFER_INFO

    invoke GetStdHandle,STD_OUTPUT_HANDLE
    mov hOutPut, eax

    invoke GetConsoleScreenBufferInfo,hOutPut,ADDR sbi

    mov eax, sbi.dwSize     ; 2 word values returned for screen size

  ; -----------------------------------------------
  ; extract the 2 values and multiply them together
  ; -----------------------------------------------
    mov ecx, eax
    shr eax, 16
    mul cx
    mov cnt, eax

    invoke FillConsoleOutputCharacter, hOutPut, 32, cnt, NULL, ADDR noc

    movzx ecx,sbi.wAttributes
    invoke FillConsoleOutputAttribute, hOutPut, ecx, cnt, NULL, ADDR noc

    invoke SetConsoleCursorPosition, hOutPut, NULL

    ret

ClearScreen endp

; ----------------------------------------------------------------------------
;  The program waits for a key pressed
; ----------------------------------------------------------------------------
;  Returns: key code in buffer.KeyEvent.wVirtualKeyCode WORD size 
; -------------------------------------------------------------------------
;  In final release to recode with Switch Block Syntax for easy maintenance
;  and reading.
; -------------------------------------------------------------------------
;     switch uMsg
;      case WM_INITDIALOG
;         invoke SendMessage,hWin,WM_SETICON,1, FUNC(LoadIcon,NULL,IDI_ASTERISK)
;         m2m hWnd, hWin
;         return 1
;     
;       case WM_COMMAND
;         switch wParam
;           case IDOK
;             fn MessageBox,hWnd,"You clicked OK","Hi",MB_OK or MB_ICONINFORMATION
;     
;           case IDCANCEL
;             jmp quit_dialog
;         endsw
;     
;       case WM_CLOSE
;         quit_dialog:
;         invoke EndDialog,hWin,0
;     endsw
; -------------------------------------------------------------------------
;     switch variable
;       case value1 ; there must be at least one (1) case statement
;         ; do something here
;       case value2 ; optional extra case statement(s)
;         ; do something else
;       default ; optional default processing
;         ; do any default processing
;     endsw
;     
; ----------------------------------------------------------------------------------


AnyKey PROC

again:  

    INVOKE ReadConsoleInput,rHnd,offset buffer,1,offset howmany
    cmp buffer.EventType,KEY_EVENT
    jnz again

    cmp buffer.KeyEvent.bKeyDown,0
    jz again

    mov ax, buffer.KeyEvent.wVirtualKeyCode

    cmp al, E_Key
    jz  ClosePgm

    cmp al, e_Key
    jz  ClosePgm

    .IF ( al == VK_ESCAPE )

        mov ConfirmExit, YES

        jmp  BackToMain

    .ENDIF
    

    .IF ( al == R_Key || al == r_Key )

        mov RunAgain, YES

        mov AlgoColTicks, ColTicksInit

        mov  AlgoRow, AlgoRowInit

        INVOKE DisplayArea, offset TestAreaBuffer, TestAreaRowInit, TestAreaColInit, 
                            TestAreaRowEnd, TestAreaColEnd

        jmp  BackToMain

    .ENDIF


    cmp al, S_Key
    jz  S_Key_Press

    cmp al, s_Key
    jz  S_Key_Press

    cmp al, V_Key
    jz  V_Key_Press

    cmp al, v_Key
    jz  V_Key_Press

    cmp al, C_Key
    jz  C_Key_Press

    cmp al, c_Key
    jz  C_Key_Press

    cmp al, H_Key
    jz  F1_Key_Press

    cmp al, h_Key
    jz  F1_Key_Press

    cmp al, I_Key
    jz  I_Key_Press

    cmp al, i_Key
    jz  I_Key_Press

    cmp al, VK_F1
    jz  F1_Key_Press

    jmp again


C_Key_Press:

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMessage 

    jmp again

    
I_Key_Press:

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMessage 

    jmp again

S_Key_Press:

    CALL   SaveScreenFile

    jmp again

F1_Key_Press:

    INVOKE SaveScreen, offset SavedScreen

    INVOKE DisplayFmt, offset HelpScreen

    jmp WaitingForAKey


; -------------------------------------------------------------
;    Mov DisplayFlag, NO
;
;    jmp WaitingForAKey
; -------------------------------------------------------------

V_Key_Press:

    CALL   ReadScreenFile

; -------------------------------------------------------------
;    mov    DisplayFlag, YES
; -------------------------------------------------------------

WaitingForAKey:  

    INVOKE ReadConsoleInput,rHnd,offset buffer,1,offset howmany
    cmp buffer.EventType,KEY_EVENT
    jnz WaitingForAKey

    cmp buffer.KeyEvent.bKeyDown,0
    jz WaitingForAKey

; -------------------------------------------------------------
;    cmp DisplayFlag, NO
;
;    jne  DisplaySaved
;
;    Mov DisplayFlag, YES   
;
;    jmp again
;
; -------------------------------------------------------------

DisplaySaved:

    INVOKE DisplayFmt, offset SavedScreen

    jmp again


ClosePgm:

    CALL PopBoxExit

    cmp  ConfirmExit, YES

    jnz  again

BackToMain:

    ret 

AnyKey ENDP

; -------------------------------------------------------------
; Wait for a key before continuing
; -------------------------------------------------------------

Waiting PROC  

StillWaiting:

    INVOKE ReadConsoleInput,rHnd,offset buffer,1,offset howmany
    cmp buffer.EventType,KEY_EVENT
    jnz StillWaiting

    cmp buffer.KeyEvent.bKeyDown,0
    jz StillWaiting
    
    ret 

Waiting ENDP

; -------------------------------------------------------------------------
; The data read from the file is converted into CHAR_INFO style. 
; The Console Buffer is filled with the content of the File Buffer, padding 
; a zero after each byte making it Windows Console API displayable. 
; -------------------------------------------------------------------------

ConvertToWin PROC DestBuffer:DWORD, SourceBuffer:DWORD, NumCyclesToDo:DWORD


    mov  esi, SourceBuffer
    mov  edi, DestBuffer

    mov  ecx, NumCyclesToDo
    xor  eax, eax

    xor  ebx, ebx

    bt   ecx, 0
    jc   IsOdd
    mov  OddNumber, 0
    jmp  NextChar

IsOdd:

    mov OddNumber, 1
    
NextChar:


    mov eax, [esi]
    mov [edi], al
    mov [edi+2], ah

    bswap eax

    mov [edi+Four], ah    
    mov [edi+6], al

    add esi, Four
    add edi, Four*2

    dec ecx
    jnz NextChar 

; --------------------------------------------------------------------------------
;  Verify if the number of characters is odd to manage the extra-bytes
; --------------------------------------------------------------------------------

    bt  OddNumber, 0
    jnc End_cycle

    movzx eax, WORD PTR [esi]
    mov [edi], al
    mov [edi+2], ah   

End_cycle:

    ret 
    
ConvertToWin ENDP

; --------------------------------------------------------------------------------
;  Convert the byte string into word string using XMM register and punpcklbw
; --------------------------------------------------------------------------------

ConvertXmm PROC

    lea  esi, FileBuffer
    lea  edi, ConsoleScreen16
    
    mov  ecx, NumCycles

    pxor xmm7,xmm7

    shr  ecx,1

    
    @@:
    
    movq      xmm0, qword ptr [esi]
    punpcklbw xmm0, xmm7
    add       esi,  Eight
    movdqa    [edi],xmm0
    add  edi, (Eight * Two)
    dec  ecx
    jnz  @B
    
    
    ret    

ConvertXmm ENDP


; -------------------------------------------------------------------------
; The data read from the Screen Buffer is converted into DOS style. 
; The New Buffer is filled with the content of the Screen Buffer, removing 
; a char after each byte making it DOS compatible. 
; -------------------------------------------------------------------------

ConvertToDOS PROC DestBuffer:DWORD, SourceBuffer:DWORD


    mov  esi, SourceBuffer
    mov  edi, DestBuffer

    mov  ecx, CharNumber
    xor  eax, eax

    xor  ebx, ebx

NextChar:

    mov eax, [esi]
    mov [edi], al
    shr eax, N16
    mov [edi+1], al

    add esi, Four
    add edi, Two

    dec ecx
    jnz NextChar 

End_cycle:

    ret 
    
ConvertToDOS ENDP


; --------------------------------------------------------------------------------
; Set the attribute colors for next PRINT/DISPLAY instruction
;----------------------------------------------------------------------------


SetColorTo PROC Fore:DWORD, Back:DWORD

    xor  eax, eax

    mov  eax, Back
    shl  eax, Four
    add  eax, Fore
    mov  ChosenColor, eax 

    invoke SetConsoleTextAttribute, wHnd, ChosenColor

    ret

SetColorTo ENDP

; --------------------------------------------------------------------------------
; A string is displayed at Column and Row for the passed Lenght
; ----------------------------------------------------------------------------

DisplayAt PROC x:DWORD, y:DWORD, StringBuffer:DWORD, lenght:DWORD
   
    LOCAL n_of_chars:DWORD
    LOCAL position:COORD
   
    mov eax, x
    mov position.x, ax
    mov eax, y
    mov position.y, ax
   
    invoke SetConsoleCursorPosition, wHnd, DWORD PTR position
    invoke WriteConsole, wHnd, StringBuffer, lenght, ADDR n_of_chars, NULL
   
    ret
    
DisplayAt ENDP


; ----------------------------------------------------------------------------


;----------------------------------------------------------------------------
; Pop UP a Message Box to display some stuff
;----------------------------------------------------------------------------

PopBox PROC PtrBoxTitle:DWORD, PtrBoxMessage:DWORD 


    invoke MessageBox, NULL, PtrBoxMessage,  PtrBoxTitle,
                       MB_OK  OR MB_SYSTEMMODAL OR MB_ICONINFORMATION 

    ret

PopBox ENDP

;----------------------------------------------------------------------------
; Pop UP a Message Box to confirm exiting the program
;----------------------------------------------------------------------------

PopBoxExit PROC


    invoke MessageBox, NULL, offset ExitMessage,  offset ExitTitle,
                       MB_OK  OR MB_SYSTEMMODAL OR MB_ICONQUESTION OR MB_YESNO 

    mov ButtonClicked, eax

    cmp ButtonClicked, OK

    jnz GoBack

    mov ConfirmExit, YES
    
GoBack:  

    ret

PopBoxExit ENDP

;----------------------------------------------------------------------------
; Choose a saved Screen File to Read/Display
;----------------------------------------------------------------------------

ReadScreenFile PROC

    mov PtrFileName, OpenFileDlg (0,0, addr CaptionRead, addr szFilter) 

    cmp BYTE PTR [eax], 0 ; test if 1st byte is zero [No selection made]

    je  Finish

    INVOKE ReadChosenFile, DWORD PTR PtrFileName, ADDR FileBuffer2

    INVOKE ConvertToWin, ADDR FileScreen, ADDR FileBuffer2, NumCycles

    INVOKE DisplayFmt, offset FileScreen

    INVOKE SetColorTo, BrightWhite, DarkRed

    lea    eax, TextString
    xor    ebx, ebx

    mov    [eax], ebx

    mov    PtrTextString, offset TextString

    mov    PtrTextString, add$(PtrTextString, " Saved Screen File - Press any key to Return to Main ")    

    INVOKE DisplayAt, 20, 39, ADDR TextString, 53  

Finish:

    ret

ReadScreenFile ENDP  

;----------------------------------------------------------------------------
; Choose a name to save actual Screen to a Disk File
;----------------------------------------------------------------------------

SaveScreenFile PROC

    INVOKE SaveScreen, OFFSET ResultScreen

    INVOKE ConvertToDOS, ADDR FileBuffer2, ADDR ResultScreen 

    mov PtrFileName, SaveFileDlg (0,0, addr CaptionSave, addr szFilter) 

    cmp BYTE PTR [eax], 0 ; test if 1st byte is zero [No selection made]

    je  NoSave

    INVOKE SaveChosenFile, DWORD PTR PtrFileName, OFFSET FileBuffer2

NoSave:

    ret

SaveScreenFile ENDP  

; ---------------------------------------------------------------------------
; Read the Screen File.
; ---------------------------------------------------------------------------
; 1] The initial 4 bytes contain:
;
;    RowInitial, RowFinal, ColInitial, ColFinal of the Screen File to display
;
; 2] The remaining bytes contain the pairs of Characters/Attributes of
;
;    the Screen File in DOS style.
; ---------------------------------------------------------------------------
 

ReadChosenFile PROC szFmtName:DWORD, PtrBuf:DWORD 

    mov hFile, fopen(szFmtName)

    mov ebx,   fread(hFile, ADDR RowInitialFile, Four)

    mov ebx,   fread(hFile, PtrBuf, BytesToRead)

    fclose hFile

    ret 
    
ReadChosenFile ENDP  

; ---------------------------------------------------------------------------
; Save the Screen to a disk File.
; ---------------------------------------------------------------------------
; 1] The initial 4 bytes contain:
;
;    RowInitial, RowFinal, ColInitial, ColFinal of the Screen File to display
;
; 2] The remaining bytes contain the pairs of Characters/Attributes of
;
;    the Screen File in DOS style.
; ---------------------------------------------------------------------------
 

SaveChosenFile PROC szFmtName:DWORD, PtrBuf:DWORD 

    mov hFile,  fcreate(szFmtName)

WriteFour:

    mov ebx,   fwrite(hFile, ADDR RowInitialFile, Four)

    cmp ebx,   Four

    je  WriteScreen

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMsgErrFour 

    jmp EndSave  

WriteScreen:

    mov ebx,   fwrite(hFile, PtrBuf, BytesToRead)

    cmp ebx,   BytesToRead

    je  SaveOK

SaveError:

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMsgErrSave 

    jmp EndSave   

SaveOK:

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMsgSaved 
    
EndSave:

    fclose hFile

    ret 
    
SaveChosenFile ENDP  

; --------------------------------------------------------------------------------
; Check for Windows Version - GregL updated version to detect Win 7
; -------------------------------------------------------------------------

GetWindowsOS PROC pszOS:PTR BYTE

    LOCAL hMod:DWORD
    LOCAL dwType:DWORD
    LOCAL pGNSI:PTR
    LOCAL pGPI:PTR
    LOCAL osvi:OSVERSIONINFOEX_NEW
    LOCAL sysinfo:SYSTEM_INFO
    LOCAL szBuild[N16]:BYTE
    LOCAL szBuf[128]:BYTE
    
    INVOKE RtlZeroMemory, ADDR osvi, SIZEOF OSVERSIONINFOEX
    INVOKE RtlZeroMemory, ADDR sysinfo, SIZEOF SYSTEM_INFO
    mov osvi.dwOSVersionInfoSize, SIZEOF OSVERSIONINFOEX
    INVOKE GetVersionEx, ADDR osvi
    ; Call GetNativeSystemInfo if supported or GetSystemInfo otherwise.
    INVOKE GetModuleHandle, CTEXT("kernel32.dll")
    mov hMod, eax
    INVOKE GetProcAddress, hMod, CTEXT("GetNativeSystemInfo")
    mov pGNSI, eax
    .IF pGNSI != NULL
        lea eax, sysinfo
        push eax
        call pGNSI
    .ELSE
        INVOKE GetSystemInfo, ADDR sysinfo
    .ENDIF  
    
    .IF (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) && (osvi.dwMajorVersion > 3)
        ; This program does not support dwMajorVersion less than 4
        INVOKE lstrcpy, pszOS, CTEXT("Microsoft ")
        .IF (osvi.dwMajorVersion == 6)
            .IF (osvi.dwMinorVersion == 0)
                ; Windows Vista or Windows Server 2008
                .IF osvi.wProductType == VER_NT_WORKSTATION
                    INVOKE lstrcat, pszOS, CTEXT("Windows Vista ")
                .ELSE
                    INVOKE lstrcat, pszOS, CTEXT("Windows Server 2008")
                .ENDIF
            .ELSEIF (osvi.dwMinorVersion == 1)
                ; Windows 7 or Windows Server 2008 R2
                .IF osvi.wProductType == VER_NT_WORKSTATION
                    INVOKE lstrcat, pszOS, CTEXT("Windows 7 ")
                .ELSE
                    INVOKE lstrcat, pszOS, CTEXT("Windows Server 2008 R2")
                .ENDIF
            .ENDIF
            
            INVOKE GetProcAddress, hMod, CTEXT("GetProductInfo")
            mov pGPI, eax
            lea eax, dwType
            push eax
            push 0
            push 0
            push osvi.dwMinorVersion
            push osvi.dwMajorVersion
            call pGPI
            .IF dwType == PRODUCT_ULTIMATE
               INVOKE lstrcat, pszOS, CTEXT("Ultimate Edition")
            .ELSEIF dwType == PRODUCT_PROFESSIONAL
               INVOKE lstrcat, pszOS, CTEXT("Professional")               
            .ELSEIF dwType == PRODUCT_HOME_PREMIUM
               INVOKE lstrcat, pszOS, CTEXT("Home Premium Edition")
            .ELSEIF dwType == PRODUCT_HOME_BASIC
               INVOKE lstrcat, pszOS, CTEXT("Home Basic Edition")
            .ELSEIF dwType == PRODUCT_ENTERPRISE
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition")
            .ELSEIF dwType == PRODUCT_BUSINESS
               INVOKE lstrcat, pszOS, CTEXT("Business Edition")
            .ELSEIF dwType == PRODUCT_STARTER
               INVOKE lstrcat, pszOS, CTEXT("Starter Edition")
            .ELSEIF dwType == PRODUCT_CLUSTER_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Cluster Server Edition")
            .ELSEIF dwType == PRODUCT_DATACENTER_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition")
            .ELSEIF dwType == PRODUCT_DATACENTER_SERVER_CORE
               INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition (core installation)")
            .ELSEIF dwType == PRODUCT_ENTERPRISE_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition")
            .ELSEIF dwType == PRODUCT_ENTERPRISE_SERVER_CORE
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition (core installation)")
            .ELSEIF dwType == PRODUCT_ENTERPRISE_SERVER_IA64
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition for Itanium-based Systems")
            .ELSEIF dwType == PRODUCT_SMALLBUSINESS_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Small Business Server")
            .ELSEIF dwType == PRODUCT_SMALLBUSINESS_SERVER_PREMIUM
               INVOKE lstrcat, pszOS, CTEXT("Small Business Server Premium Edition")
            .ELSEIF dwType == PRODUCT_STANDARD_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Standard Edition")
            .ELSEIF dwType == PRODUCT_STANDARD_SERVER_CORE
               INVOKE lstrcat, pszOS, CTEXT("Standard Edition (core installation)")
            .ELSEIF dwType == PRODUCT_WEB_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Web Server Edition")
            .ENDIF    
            .IF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64
                INVOKE lstrcat, pszOS, CTEXT( ", 64-bit" )
            .ELSEIF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL
                INVOKE lstrcat, pszOS, CTEXT(", 32-bit")
            .ENDIF
        .ELSEIF (osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion == 2)
            ; Windows Server 2003    
            INVOKE GetSystemMetrics, SM_SERVERR2
            .IF eax
                INVOKE lstrcat, pszOS, CTEXT("Windows Server 2003 R2, ")
            .ELSEIF osvi.wSuiteMask == VER_SUITE_STORAGE_SERVER    
                INVOKE lstrcat, pszOS, CTEXT( "Windows Storage Server 2003")
            .ELSEIF (osvi.wProductType == VER_NT_WORKSTATION) && (sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64)
                INVOKE lstrcat, pszOS, CTEXT( "Windows XP Professional x64 Edition")
            .ELSE
                INVOKE lstrcat, pszOS, CTEXT("Windows Server 2003, ")
            .ENDIF
            .IF osvi.wProductType != VER_NT_WORKSTATION
                .IF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_IA64
                    .IF osvi.wSuiteMask & VER_SUITE_DATACENTER
                       INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition for Itanium-based Systems")
                    .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                       INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition for Itanium-based Systems")
                    .ENDIF   
                .ENDIF
            .ELSEIF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64
                .IF osvi.wSuiteMask & VER_SUITE_DATACENTER
                    INVOKE lstrcat, pszOS, CTEXT("Datacenter x64 Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                    INVOKE lstrcat, pszOS, CTEXT("Enterprise x64 Edition")
                .ELSE 
                    INVOKE lstrcat, pszOS, CTEXT("Standard x64 Edition")
                .ENDIF    
            .ELSE
                .IF osvi.wSuiteMask & VER_SUITE_COMPUTE_SERVER
                    INVOKE lstrcat, pszOS, CTEXT("Compute Cluster Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_DATACENTER
                    INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                    INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_BLADE
                    INVOKE lstrcat, pszOS, CTEXT("Web Edition")
                .ELSE 
                    INVOKE lstrcat, pszOS, CTEXT("Standard Edition")
                .ENDIF    
            .ENDIF
        .ELSEIF (osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion == 1)
            ; Windows XP            
            INVOKE lstrcat, pszOS, CTEXT("Windows XP ")
            .IF osvi.wSuiteMask & VER_SUITE_PERSONAL
                INVOKE lstrcat, pszOS, CTEXT("Home Edition")
            .ELSE 
                INVOKE lstrcat, pszOS, CTEXT("Professional")
            .ENDIF    
        .ELSEIF (osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion == 0)
            ; Windows 2000
            INVOKE lstrcat, pszOS, CTEXT("Windows 2000 ")
             .IF osvi.wProductType == VER_NT_WORKSTATION
                 INVOKE lstrcat, pszOS, CTEXT("Professional" )
             .ELSE
                 .IF osvi.wSuiteMask & VER_SUITE_DATACENTER
                     INVOKE lstrcat, pszOS, CTEXT("Datacenter Server" )
                 .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                     INVOKE lstrcat, pszOS, CTEXT("Advanced Server")
                 .ELSE 
                     INVOKE lstrcat, pszOS, CTEXT("Server")
                 .ENDIF  
             .ENDIF    
        .ELSEIF osvi.dwMajorVersion == 4
            ; Windows NT 4.0
            INVOKE lstrcat, pszOS, CTEXT("Windows NT 4.0")
        .ENDIF  
        ; Include service pack (if any) and build number.
        INVOKE lstrlen, ADDR osvi.szCSDVersion 
        .IF eax > 0
            INVOKE lstrcat, pszOS, CTEXT(" ")
            INVOKE lstrcat, pszOS, ADDR osvi.szCSDVersion
        .ENDIF
        INVOKE wsprintf, ADDR szBuild, CTEXT(" (build %d)"), osvi.dwBuildNumber
        INVOKE lstrcat, pszOS, ADDR szBuild
        xor eax, eax
    .ELSE
        INVOKE lstrcpy, pszOS, 
        CTEXT("This program does not support Windows versions less than Windows NT 4.0")
        mov eax, 1
    .ENDIF
    ret
GetWindowsOS ENDP

; --------------------------------------------------------------------------------


;#############################################################################
; Instructions detection code by Alex aka Antariy
;#############################################################################

OPTION PROLOGUE:NONE
OPTION EPILOGUE:NONE

AxCPUid_Features	struct DWORD
	Is486capable	dd	?
	IsP1		dd	?
	IsP1MMX		dd	?
	IsPPro		dd	?
	IsSSE1		dd	?
	IsSSE2		dd	?
	IsSSE3		dd	?
	IsSSSE3		dd	?
	IsSSE41		dd	?
	IsSSE42		dd	?
	BrandName	db	64 dup (?)
AxCPUid_Features	ends

AxCPUidCodeSizeStart EQU $


; Return values:
; 	low order WORD of EAX:
; 		instructions supported:
; 			0 - need upgrade
; 			above 0 - then supported:
; 			1 - MMX
; 			2 - SSE1
; 			3 - SSE2
;			4 - SSE3
; 			5 - SSSE3
; 			6 - SSE4.1
; 			7 - SSE4.2
;	high order WORD of EAX:
;		size of the output string in chars, without termination null

align N16

AxCPUid_FillStructure	proc	lpAxCPUid_Features:PTR AxCPUid_Features
	.data?
		ifrunnedAndSupportCPUid	dd	?
	.code
	
	push ebp
	push ebx
	
	xor eax,eax
	mov ecx,sizeof AxCPUid_Features

	mov ebp,[esp+4+8]
	
	@@:
	mov [ebp+ecx-4].AxCPUid_Features.Is486capable,eax
	add ecx,-4
	jnz @B

	mov ebx,ifrunnedAndSupportCPUid
	dec ebx
	jz @done
	dec ebx
	jz @is486capable
	
	inc ifrunnedAndSupportCPUid
	
	pushfd
	pop ecx
	mov ebx,ecx
	xor ebx,200000h	; switch ID flag
	push ebx
	popfd		; change EFLAGS
	pushfd
	pop ebx
	xor ebx,ecx	; if previous ID bit was equal to current
	test ebx,200000h; then it would be dropped
	jz @done	; otherwise - it set
	
	inc ifrunnedAndSupportCPUid
	
	@is486capable:

	or [ebp].AxCPUid_Features.Is486capable,1
	
	xor eax,eax
	cpuid
	test eax,eax
	jz @done
	
	mov eax,1
	cpuid
	
	shr edx,5	; has RDTSC - so, this is PI at least
	adc [ebp].AxCPUid_Features.IsP1,0
	
	shr edx,2	; has PAE - so, PPro at least
	adc [ebp].AxCPUid_Features.IsPPro,0
	
	shr edx,17	; has MMX
	adc [ebp].AxCPUid_Features.IsP1MMX,0
	
	shr edx,2	; has SSE1
	adc [ebp].AxCPUid_Features.IsSSE1,0
	
	shr edx,1	; has SSE2
	adc [ebp].AxCPUid_Features.IsSSE2,0
	
	
	
	shr ecx,1	; has SSE3
	adc [ebp].AxCPUid_Features.IsSSE3,0
	
	shr ecx,9	; has SSSE3
	adc [ebp].AxCPUid_Features.IsSSSE3,0
	
	shr ecx,10	; has SSE4.1
	adc [ebp].AxCPUid_Features.IsSSE41,0
	
	shr ecx,1	; has SSE4.2
	adc [ebp].AxCPUid_Features.IsSSE42,0
	
	
	; get CPU brand name, if exist
	

	; fix for PIII, not SSE2 capable CPU cannot have brand name
	cmp [ebp].AxCPUid_Features.IsSSE2,0
	jz @done

	mov eax,80000000h
	cpuid
	add eax,eax	; check for zero and no any extended functions
	jz @done
	
	lea ebp,[ebp].AxCPUid_Features.BrandName
	cmp eax,8	; needed at least 80000004h function
	mov eax,0
	jb @done
	
	push esi
	mov esi,80000002h
	@@:
	mov eax,esi
	cpuid
	inc esi
	mov [ebp],eax
	mov [ebp+4],ebx
	mov [ebp+8],ecx
	mov [ebp+12],edx
	add ebp, N16
	cmp esi,80000005h
	jb @B		
	pop esi
	
	or eax,1	; in case of terminated brand string
	
	@done:
	pop ebx
	pop ebp
	ret 4
AxCPUid_FillStructure	endp


; Return values:
; 	low order WORD of EAX:
; 		instructions supported:
; 			0 - need upgrade
; 			above 0 - then supported:
; 			1 - MMX
; 			2 - SSE1
; 			3 - SSE2
;			4 - SSE3
; 			5 - SSSE3
; 			6 - SSE4.1
; 			7 - SSE4.2
;	high order WORD of EAX:
;		size of the output string in chars, without termination null


align N16

AxCPUid proc lpStrOut:DWORD

	push ebx
	push esi
	push edi
	
	invoke GetStdHandle,STD_OUTPUT_HANDLE
	xchg eax,ebx
	
	mov esi,[esp+12+4]
		
	invoke AxCPUid_FillStructure,esi
	cmp [esi].AxCPUid_Features.Is486capable,0
	jnz @F	
	
	push eax
	mov edx,esp
	
	push 0
	push edx
	push sizeof @@needupgrade
	push offset @@needupgrade
	push ebx
	call WriteFile
	pop ecx
	xor eax,eax
	jmp @done
	
	@@:
	
	mov edi,[esi].AxCPUid_Features.IsP1MMX
	add edi,[esi].AxCPUid_Features.IsSSE1
	add edi,[esi].AxCPUid_Features.IsSSE2
	add edi,[esi].AxCPUid_Features.IsSSE3
	add edi,[esi].AxCPUid_Features.IsSSSE3
	add edi,[esi].AxCPUid_Features.IsSSE41
	add edi,[esi].AxCPUid_Features.IsSSE42
	
	
	comment ~
			
	mov eax,[esi].AxCPUid_Features.IsSSE42
	lea eax,[eax*2+offset @@sse42]

	mov edx,[esi].AxCPUid_Features.IsSSE41
	lea edx,[edx*2+offset @@sse41]
	
	mov ecx,[esi].AxCPUid_Features.IsSSSE3
	lea ecx,[ecx*2+offset @@ssse3]
	
	push eax
	push edx
	push ecx
	
	mov eax,[esi].AxCPUid_Features.IsSSE3
	lea eax,[eax*2+offset @@sse3]
	
	mov edx,[esi].AxCPUid_Features.IsSSE2
	lea edx,[edx*2+offset @@sse2]
	
	mov ecx,[esi].AxCPUid_Features.IsSSE1
	lea ecx,[ecx*2+offset @@sse1]
	
	push eax
	push edx
	push ecx	
	
	mov eax,[esi].AxCPUid_Features.IsP1MMX
	lea eax,[eax*2+offset @@mmx]
	
	~
	
	lea edx,[esi].AxCPUid_Features.BrandName
	cmp dword ptr [edx],0
	jnz @hasbrandname

	mov ecx,[esi].AxCPUid_Features.Is486capable
	add ecx,[esi].AxCPUid_Features.IsP1
	add ecx,[esi].AxCPUid_Features.IsP1MMX
	add ecx,[esi].AxCPUid_Features.IsPPro
	
	cmp ecx,4
	jb @F
	add ecx,[esi].AxCPUid_Features.IsP1MMX	; PII is PPro with MMX
	add ecx,[esi].AxCPUid_Features.IsSSE1	; PIII is PPro with MMX and SSE1
	@@:
		
	mov edx,[ecx*4+offset @@earlycpus]
		
	@hasbrandname:
	add edx,1
	cmp byte ptr [edx-1]," "
	jz @hasbrandname
	
	mov eax,[edi*4+@@shortdescriptions]
	push eax
	
	dec edx
	
	push eax
	push edx
	mov eax,1
	
	cpuid
	shr ebx,16
	and ebx,255
      sete al
      add bl,al  
	mov [esp+4],ebx
	push offset @@fmtFeatures
	push esi
	call wsprintf
	
	shl eax,16	; put count of chars into high word

	add esp,5*4

	lea eax,[eax+edi]	; put instructions set supported into AX
	@done:
	pop edi
	pop esi
	pop ebx
	ret 4
	
	even
	@@needupgrade	db	"This is time for upgrade indeed, i386 or early i486 :)"
	even
	@@i486		db	"Old-good 80486",0
	even
	@@p1		db	"Pentium",0
	even
	@@pmmx		db	"Pentium with MMX Technology",0
	even
	@@ppro		db	"Pentium Pro",0
	even
	@@p2		db	"Pentium II",0
	even
	@@p3		db	"Pentium III",0
	
	align 4
	@@earlycpus	dd	0
			dd	offset @@i486
			dd	offset @@p1
			dd	offset @@pmmx
			dd	offset @@ppro
			dd	offset @@p2
			dd	offset @@p3
	    
    	@@shortdescriptions	dd	offset @@needupgrade
    				dd	offset @@mmx
    				dd	offset @@sse1
    				dd	offset @@sse2
    				dd	offset @@sse3
    				dd	offset @@ssse3
    				dd	offset @@sse41
    				dd	offset @@sse42

	even
	@@fmtFeatures	db	"%s with %u logical core(s) with %s",0
	even
	@@mmx		db	"MMX",0
	even
	@@sse1		db	"SSE1",0
	even
	@@sse2		db	"SSE2",0
	even
	@@sse3		db	"SSE3",0
	even
	@@ssse3		db	"SSSE3",0
	even
	@@sse41		db	"SSE4.1",0
	even
	@@sse42		db	"SSE4.2",0
	
	
AxCPUid endp

OPTION PROLOGUE:PrologueDef
OPTION EPILOGUE:EpilogueDef
;#############################################################################

; -------------------------------------------------------------------------
; The data read from the file is converted into CHAR_INFO style. 
; ConsoleBuffer is filled with the content of FileBuffer, padding a zero
; after each byte making it Windows Console API displayable. 
; Alex version with mmx instructions and punpcklbw.
; -------------------------------------------------------------------------

 AlexConvertArea PROC DestBuffer:DWORD, SourceBuffer:DWORD, NumCyclesToDo:DWORD

    mov  esi, SourceBuffer
    mov  edi, DestBuffer
    
    mov  ecx, NumCyclesToDo
    mov  eax, NumCyclesToDo
    pxor mm7,mm7
    and ecx,1
    
    @@:
    
    movd mm0,dword ptr [esi]
    punpcklbw mm0,mm7
    add esi,4
    movq qword ptr [edi],mm0
    add edi,8
    add eax, -1
    jnz @B

    
    jecxz @ConvDone
    movzx eax,word ptr [esi]
    mov [edi],al
    mov [edi+2],ah   
        
    @ConvDone:

    ret 
    
 AlexConvertArea ENDP

